@@ -262,6 +262,8 @@ placement_gnome_to_eel (GnomeBGPlacement p)
 		return EEL_BACKGROUND_ZOOM;
 	case GNOME_BG_PLACEMENT_TILED:
 		return EEL_BACKGROUND_TILED;
+        case GNOME_BG_PLACEMENT_SPANNED:
+                return EEL_BACKGROUND_SPANNED;
 	}
 
 	return EEL_BACKGROUND_TILED;
@@ -281,6 +283,8 @@ placement_eel_to_gnome (EelBackgroundImagePlacement p)
 		return GNOME_BG_PLACEMENT_ZOOMED;
 	case EEL_BACKGROUND_TILED:
 		return GNOME_BG_PLACEMENT_TILED;
+	case EEL_BACKGROUND_SPANNED:
+		return GNOME_BG_PLACEMENT_SPANNED;
 	}
 
 	return GNOME_BG_PLACEMENT_TILED;
@@ -746,7 +750,7 @@ eel_background_set_up_widget (EelBackground *background, GtkWidget *widget)
 	GdkWindow *window;
 	gboolean in_fade;
 
-	if (!GTK_WIDGET_REALIZED (widget)) {
+	if (!gtk_widget_get_realized (widget)) {
 		return;
 	}
 
@@ -810,7 +814,7 @@ on_background_changed (EelBackground *background)
 static void
 init_fade (EelBackground *background, GtkWidget *widget)
 {
-	if (widget == NULL || !GTK_WIDGET_REALIZED (widget))
+	if (widget == NULL || !gtk_widget_get_realized (widget))
 		return;
 
 	if (!background->details->is_desktop) {
@@ -962,7 +966,7 @@ eel_background_set_desktop (EelBackground *background, GtkWidget *widget, gboole
 {
 	background->details->is_desktop = is_desktop;
 
-	if (GTK_WIDGET_REALIZED(widget) && background->details->is_desktop) {
+	if (gtk_widget_get_realized (widget) && background->details->is_desktop) {
 		widget_realized_setup (widget, background);
 	}
 	
