@@ -42,6 +42,7 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <gio/gio.h>
+#include <libnautilus-private/nautilus-clipboard-monitor.h>
 #include <libnautilus-private/nautilus-directory-background.h>
 #include <libnautilus-private/nautilus-directory.h>
 #include <libnautilus-private/nautilus-dnd.h>
@@ -108,6 +109,8 @@ struct FMIconViewDetails
 	int num_screens;
 
 	gboolean compact;
+
+	gulong clipboard_handler_id;
 };
 
 
@@ -150,6 +153,13 @@ static const SortCriterion sort_criteria[] = {
 		"Sort by Emblems",
 		N_("by _Emblems"),
 		N_("Keep icons sorted by emblems in rows")
+	},
+	{
+		NAUTILUS_FILE_SORT_BY_TRASHED_TIME,
+		"trashed",
+		"Sort by Trash Time",
+		N_("by T_rash Time"),
+		N_("Keep icons sorted by trash time in rows")
 	}
 };
 
@@ -198,6 +208,12 @@ fm_icon_view_destroy (GtkObject *object)
 		icon_view->details->react_to_icon_change_idle_id = 0;
         }
 
+	if (icon_view->details->clipboard_handler_id != 0) {
+		g_signal_handler_disconnect (nautilus_clipboard_monitor_get (),
+					     icon_view->details->clipboard_handler_id);
+		icon_view->details->clipboard_handler_id = 0;
+	}
+
 	/* kill any sound preview process that is ongoing */
 	preview_audio (icon_view, NULL, FALSE);
 
@@ -617,12 +633,14 @@ update_layout_menus (FMIconView *view)
 	gboolean is_auto_layout;
 	GtkAction *action;
 	const char *action_name;
+	NautilusFile *file;
 
 	if (view->details->icon_action_group == NULL) {
 		return;
 	}
 
 	is_auto_layout = fm_icon_view_using_auto_layout (view);
+	file = fm_directory_view_get_directory_as_file (FM_DIRECTORY_VIEW (view));
 
 	if (fm_icon_view_supports_auto_layout (view)) {
 		/* Mark sort criterion. */
@@ -643,6 +661,15 @@ update_layout_menus (FMIconView *view)
 		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
 					      view->details->sort_reversed);
 		gtk_action_set_sensitive (action, is_auto_layout);
+
+		action = gtk_action_group_get_action (view->details->icon_action_group,
+		                                      FM_ACTION_SORT_TRASH_TIME);
+
+		if (file != NULL && nautilus_file_is_in_trash (file)) {
+			gtk_action_set_visible (action, TRUE);
+		} else {
+			gtk_action_set_visible (action, FALSE);
+		}
 	}
 
 	action = gtk_action_group_get_action (view->details->icon_action_group,
@@ -1194,17 +1221,40 @@ fm_icon_view_begin_loading (FMDirectoryView *view)
 	update_layout_menus (icon_view);
 }
 
+static void
+icon_view_notify_clipboard_info (NautilusClipboardMonitor *monitor,
+                                 NautilusClipboardInfo *info,
+                                 FMIconView *icon_view)
+{
+	GList *icon_data;
+
+	icon_data = NULL;
+	if (info && info->cut) {
+		icon_data = info->files;
+	}
+
+	nautilus_icon_container_set_highlighted_for_clipboard (
+		get_icon_container (icon_view), icon_data);
+}
+
 static void
 fm_icon_view_end_loading (FMDirectoryView *view,
 			  gboolean all_files_seen)
 {
 	FMIconView *icon_view;
 	GtkWidget *icon_container;
+	NautilusClipboardMonitor *monitor;
+	NautilusClipboardInfo *info;
 
 	icon_view = FM_ICON_VIEW (view);
 
 	icon_container = GTK_WIDGET (get_icon_container (icon_view));
 	nautilus_icon_container_end_loading (NAUTILUS_ICON_CONTAINER (icon_container), all_files_seen);
+
+	monitor = nautilus_clipboard_monitor_get ();
+	info = nautilus_clipboard_monitor_get_clipboard_info (monitor);
+
+	icon_view_notify_clipboard_info (monitor, info, icon_view);
 }
 
 static NautilusZoomLevel
@@ -1494,8 +1544,8 @@ fm_icon_view_start_renaming_file (FMDirectoryView *view,
 static const GtkActionEntry icon_view_entries[] = {
   /* name, stock id, label */  { "Arrange Items", NULL, N_("Arran_ge Items") }, 
   /* name, stock id */         { "Stretch", NULL,
-  /* label, accelerator */       N_("Stretc_h Icon..."), NULL,
-  /* tooltip */                  N_("Make the selected icon stretchable"),
+  /* label, accelerator */       N_("Resize Icon..."), NULL,
+  /* tooltip */                  N_("Make the selected icon resizable"),
                                  G_CALLBACK (action_stretch_callback) },
   /* name, stock id */         { "Unstretch", NULL,
   /* label, accelerator */       N_("Restore Icons' Original Si_zes"), NULL,
@@ -1550,6 +1600,10 @@ static const GtkRadioActionEntry arrange_radio_entries[] = {
     N_("By _Emblems"), NULL,
     N_("Keep icons sorted by emblems in rows"),
     NAUTILUS_FILE_SORT_BY_EMBLEMS },
+  { "Sort by Trash Time", NULL,
+    N_("By T_rash Time"), NULL,
+    N_("Keep icons sorted by trash time in rows"),
+    NAUTILUS_FILE_SORT_BY_TRASHED_TIME },
 };
 
 static void
@@ -3015,6 +3069,11 @@ fm_icon_view_init (FMIconView *icon_view)
 				 G_CALLBACK (icon_view_handle_text), icon_view, 0);
 	g_signal_connect_object (get_icon_container (icon_view), "handle_raw",
 				 G_CALLBACK (icon_view_handle_raw), icon_view, 0);
+
+	icon_view->details->clipboard_handler_id = 
+		g_signal_connect (nautilus_clipboard_monitor_get (),
+		                  "clipboard_info",
+		                  G_CALLBACK (icon_view_notify_clipboard_info), icon_view);
 }
 
 static NautilusView *
