@@ -54,8 +54,9 @@ static guint list_model_signals[LAST_SIGNAL] = { 0 };
 static int fm_list_model_file_entry_compare_func (gconstpointer a,
 						  gconstpointer b,
 						  gpointer      user_data);
-
-static GObjectClass *parent_class;
+static void fm_list_model_tree_model_init (GtkTreeModelIface *iface);
+static void fm_list_model_sortable_init (GtkTreeSortableIface *iface);
+static void fm_list_model_multi_drag_source_init (EggTreeMultiDragSourceIface *iface);
 
 struct FMListModelDetails {
 	GSequence *files;
@@ -74,6 +75,8 @@ struct FMListModelDetails {
 	int drag_begin_y;
 
 	GPtrArray *columns;
+
+	GList *highlight_files;
 };
 
 typedef struct {
@@ -94,6 +97,14 @@ struct FileEntry {
 	guint loaded : 1;
 };
 
+G_DEFINE_TYPE_WITH_CODE (FMListModel, fm_list_model, G_TYPE_OBJECT,
+			 G_IMPLEMENT_INTERFACE (GTK_TYPE_TREE_MODEL,
+						fm_list_model_tree_model_init)
+			 G_IMPLEMENT_INTERFACE (GTK_TYPE_TREE_SORTABLE,
+						fm_list_model_sortable_init)
+			 G_IMPLEMENT_INTERFACE (EGG_TYPE_TREE_MULTI_DRAG_SOURCE,
+						fm_list_model_multi_drag_source_init));
+
 static const GtkTargetEntry drag_types [] = {
 	{ NAUTILUS_ICON_DND_GNOME_ICON_LIST_TYPE, 0, NAUTILUS_ICON_DND_GNOME_ICON_LIST },
 	{ NAUTILUS_ICON_DND_URI_LIST_TYPE, 0, NAUTILUS_ICON_DND_URI_LIST },
@@ -244,7 +255,7 @@ fm_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int column
 	FileEntry *file_entry;
 	NautilusFile *file;
 	char *str;
-	GdkPixbuf *icon;
+	GdkPixbuf *icon, *rendered_icon;
 	int icon_size;
 	guint emblem_size;
 	NautilusZoomLevel zoom_level;
@@ -287,7 +298,8 @@ fm_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int column
 			icon_size = nautilus_get_icon_size_for_zoom_level (zoom_level);
 
 			flags = NAUTILUS_FILE_ICON_FLAGS_USE_THUMBNAILS |
-				NAUTILUS_FILE_ICON_FLAGS_FORCE_THUMBNAIL_SIZE;
+				NAUTILUS_FILE_ICON_FLAGS_FORCE_THUMBNAIL_SIZE |
+				NAUTILUS_FILE_ICON_FLAGS_USE_MOUNT_ICON_AS_EMBLEM;
 			if (model->details->drag_view != NULL) {
 				GtkTreePath *path_a, *path_b;
 				
@@ -307,7 +319,19 @@ fm_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int column
 			}
 
 			icon = nautilus_file_get_icon_pixbuf (file, icon_size, TRUE, flags);
-  
+
+			if (model->details->highlight_files != NULL &&
+			    g_list_find_custom (model->details->highlight_files,
+			                        file, (GCompareFunc) nautilus_file_compare_location))
+			{
+				rendered_icon = eel_gdk_pixbuf_render (icon, 1, 255, 255, 0, 0);
+
+				if (rendered_icon != NULL) {
+					g_object_unref (icon);
+					icon = rendered_icon;
+				}
+			}
+
 			g_value_set_object (value, icon);
 			g_object_unref (icon);
 		}
@@ -1532,8 +1556,8 @@ fm_list_model_dispose (GObject *object)
 		g_hash_table_destroy (model->details->directory_reverse_map);
 		model->details->directory_reverse_map = NULL;
 	}
-	
-	EEL_CALL_PARENT (G_OBJECT_CLASS, dispose, (object));
+
+	G_OBJECT_CLASS (fm_list_model_parent_class)->dispose (object);
 }
 
 static void
@@ -1543,9 +1567,14 @@ fm_list_model_finalize (GObject *object)
 
 	model = FM_LIST_MODEL (object);
 
+	if (model->details->highlight_files != NULL) {
+		nautilus_file_list_free (model->details->highlight_files);
+		model->details->highlight_files = NULL;
+	}
+
 	g_free (model->details);
-	
-	EEL_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
+
+	G_OBJECT_CLASS (fm_list_model_parent_class)->finalize (object);
 }
 
 static void
@@ -1570,8 +1599,6 @@ fm_list_model_class_init (FMListModelClass *klass)
 	attribute_date_modified_q = g_quark_from_static_string ("date_modified");
 	
 	object_class = (GObjectClass *)klass;
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->finalize = fm_list_model_finalize;
 	object_class->dispose = fm_list_model_dispose;
 
@@ -1619,57 +1646,6 @@ fm_list_model_multi_drag_source_init (EggTreeMultiDragSourceIface *iface)
 	iface->drag_data_delete = fm_list_model_multi_drag_data_delete;
 }
 
-GType
-fm_list_model_get_type (void)
-{
-	static GType object_type = 0;
-
-	if (object_type == 0) {
-		const GTypeInfo object_info = {
-			sizeof (FMListModelClass),
-			NULL,		/* base_init */
-			NULL,		/* base_finalize */
-			(GClassInitFunc) fm_list_model_class_init,
-			NULL,		/* class_finalize */
-			NULL,		/* class_data */
-			sizeof (FMListModel),
-			0,
-			(GInstanceInitFunc) fm_list_model_init,
-		};
-
-		const GInterfaceInfo tree_model_info = {
-			(GInterfaceInitFunc) fm_list_model_tree_model_init,
-			NULL,
-			NULL
-		};
-
-		const GInterfaceInfo sortable_info = {
-			(GInterfaceInitFunc) fm_list_model_sortable_init,
-			NULL,
-			NULL
-		};
-
-		const GInterfaceInfo multi_drag_source_info = {
-			(GInterfaceInitFunc) fm_list_model_multi_drag_source_init,
-			NULL,
-			NULL
-		};
-		
-		object_type = g_type_register_static (G_TYPE_OBJECT, "FMListModel", &object_info, 0);
-		g_type_add_interface_static (object_type,
-					     GTK_TYPE_TREE_MODEL,
-					     &tree_model_info);
-		g_type_add_interface_static (object_type,
-					     GTK_TYPE_TREE_SORTABLE,
-					     &sortable_info);
-		g_type_add_interface_static (object_type,
-					     EGG_TYPE_TREE_MULTI_DRAG_SOURCE,
-					     &multi_drag_source_info);
-	}
-	
-	return object_type;
-}
-
 void
 fm_list_model_subdirectory_done_loading (FMListModel *model, NautilusDirectory *directory)
 {
@@ -1711,3 +1687,45 @@ fm_list_model_subdirectory_done_loading (FMListModel *model, NautilusDirectory *
 		}
 	}
 }
+
+static void
+refresh_row (gpointer data,
+             gpointer user_data)
+{
+	NautilusFile *file;
+	FMListModel *model;
+	GList *iters, *l;
+	GtkTreePath *path;
+
+	model = user_data;
+	file = data;
+
+	iters = fm_list_model_get_all_iters_for_file (model, file);
+	for (l = iters; l != NULL; l = l->next) {
+		path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), l->data);
+		gtk_tree_model_row_changed (GTK_TREE_MODEL (model), path, l->data);
+
+		gtk_tree_path_free (path);
+	}
+
+	eel_g_list_free_deep (iters);
+}
+
+void
+fm_list_model_set_highlight_for_files (FMListModel *model,
+                                       GList *files)
+{
+	if (model->details->highlight_files != NULL) {
+		g_list_foreach (model->details->highlight_files,
+		                refresh_row, model);
+		nautilus_file_list_free (model->details->highlight_files);
+		model->details->highlight_files = NULL;
+	}
+
+	if (files != NULL) {
+		model->details->highlight_files = nautilus_file_list_copy (files);
+		g_list_foreach (model->details->highlight_files,
+		                refresh_row, model);
+
+	}
+}
