@@ -33,6 +33,7 @@
 #include "fm-list-model.h"
 #include <string.h>
 #include <eel/eel-vfs-extensions.h>
+#include <eel/eel-gdk-extensions.h>
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-gtk-macros.h>
 #include <gdk/gdk.h>
@@ -42,6 +43,7 @@
 #include <glib/gi18n.h>
 #include <glib-object.h>
 #include <libnautilus-extension/nautilus-column-provider.h>
+#include <libnautilus-private/nautilus-clipboard-monitor.h>
 #include <libnautilus-private/nautilus-column-chooser.h>
 #include <libnautilus-private/nautilus-column-utilities.h>
 #include <libnautilus-private/nautilus-debug-log.h>
@@ -104,6 +106,8 @@ struct FMListViewDetails {
 	gboolean rename_done;
 	guint renaming_file_activate_timeout;
 
+	gulong clipboard_handler_id;
+
 	GQuark last_sort_attr;
 };
 
@@ -155,6 +159,14 @@ G_DEFINE_TYPE_WITH_CODE (FMListView, fm_list_view, FM_TYPE_DIRECTORY_VIEW,
 			 G_IMPLEMENT_INTERFACE (NAUTILUS_TYPE_VIEW,
 						fm_list_view_iface_init));
 
+static const char * default_trash_visible_columns[] = {
+	"name", "size", "type", "trashed_on", "trash_orig_path", NULL
+};
+
+static const char * default_trash_columns_order[] = {
+	"name", "size", "type", "trashed_on", "trash_orig_path", NULL
+};
+
 /* for EEL_CALL_PARENT */
 #define parent_class fm_list_view_parent_class
 
@@ -1258,15 +1270,19 @@ apply_columns_settings (FMListView *list_view,
 			char **visible_columns)
 {
 	GList *all_columns;
+	NautilusFile *file;
 	GList *old_view_columns, *view_columns;
 	GHashTable *visible_columns_hash;
 	GtkTreeViewColumn *prev_view_column;
 	GList *l;
 	int i;
 
+	file = fm_directory_view_get_directory_as_file (FM_DIRECTORY_VIEW (list_view));
+
 	/* prepare ordered list of view columns using column_order and visible_columns */
 	view_columns = NULL;
-	all_columns = nautilus_get_all_columns ();
+
+	all_columns = nautilus_get_columns_for_file (file);
 	all_columns = nautilus_sort_columns (all_columns, column_order);
 
 	/* hash table to lookup if a given column should be visible */
@@ -1606,7 +1622,13 @@ get_visible_columns (FMListView *list_view)
 		g_list_free (visible_columns);
 	}
 
-	return ret ? ret : g_strdupv (default_visible_columns_auto_value);
+	if (ret != NULL) {
+		return ret;
+	}
+
+	return nautilus_file_is_in_trash (file) ?
+		g_strdupv ((gchar **) default_trash_visible_columns) :
+		g_strdupv (default_visible_columns_auto_value);
 }
 
 static char **
@@ -1638,7 +1660,13 @@ get_column_order (FMListView *list_view)
 		g_list_free (column_order);
 	}
 
-	return ret ? ret : g_strdupv (default_visible_columns_auto_value);
+	if (ret != NULL) {
+		return ret;
+	}
+
+	return nautilus_file_is_in_trash (file) ?
+		g_strdupv ((gchar **) default_trash_columns_order) :
+		g_strdupv (default_column_order_auto_value);
 }
 
 static void
@@ -2127,6 +2155,23 @@ column_chooser_changed_callback (NautilusColumnChooser *chooser,
 	g_strfreev (column_order);
 }
 
+static void
+column_chooser_set_from_arrays (NautilusColumnChooser *chooser,
+                                FMListView *view,
+                                char **visible_columns,
+                                char **column_order)
+{
+	g_signal_handlers_block_by_func 
+		(chooser, G_CALLBACK (column_chooser_changed_callback), view);
+
+	nautilus_column_chooser_set_settings (chooser,
+					      visible_columns, 
+					      column_order);
+
+	g_signal_handlers_unblock_by_func 
+		(chooser, G_CALLBACK (column_chooser_changed_callback), view);
+}
+
 static void
 column_chooser_set_from_settings (NautilusColumnChooser *chooser,
 				  FMListView *view)
@@ -2134,21 +2179,14 @@ column_chooser_set_from_settings (NautilusColumnChooser *chooser,
 	char **visible_columns;
 	char **column_order;
 
-	g_signal_handlers_block_by_func 
-		(chooser, G_CALLBACK (column_chooser_changed_callback), view);
-
 	visible_columns = get_visible_columns (view);
 	column_order = get_column_order (view);
 
-	nautilus_column_chooser_set_settings (chooser,
-					      visible_columns, 
-					      column_order);
+	column_chooser_set_from_arrays (chooser, view,
+	                                visible_columns, column_order);
 
 	g_strfreev (visible_columns);
 	g_strfreev (column_order);
-
-	g_signal_handlers_unblock_by_func 
-		(chooser, G_CALLBACK (column_chooser_changed_callback), view);
 }
 
 static void
@@ -2156,6 +2194,8 @@ column_chooser_use_default_callback (NautilusColumnChooser *chooser,
 				     FMListView *view)
 {
 	NautilusFile *file;
+	char **default_columns;
+	char **default_order;
 
 	file = fm_directory_view_get_directory_as_file 
 		(FM_DIRECTORY_VIEW (view));
@@ -2163,8 +2203,23 @@ column_chooser_use_default_callback (NautilusColumnChooser *chooser,
 	nautilus_file_set_metadata_list (file, NAUTILUS_METADATA_KEY_LIST_VIEW_COLUMN_ORDER, NULL);
 	nautilus_file_set_metadata_list (file, NAUTILUS_METADATA_KEY_LIST_VIEW_VISIBLE_COLUMNS, NULL);
 
-	set_columns_settings_from_metadata_and_preferences (FM_LIST_VIEW (view));
-	column_chooser_set_from_settings (chooser, view);
+	/* set view values ourselves, as new metadata could not have been
+	 * updated yet.
+	 */
+	default_columns = nautilus_file_is_in_trash (file) ?
+		g_strdupv ((gchar **) default_trash_visible_columns) :
+		g_strdupv (default_visible_columns_auto_value);
+
+	default_order = nautilus_file_is_in_trash (file) ?
+		g_strdupv ((gchar **) default_trash_columns_order) :
+		g_strdupv (default_column_order_auto_value);
+
+	apply_columns_settings (view, default_order, default_columns);
+	column_chooser_set_from_arrays (chooser, view,
+	                                default_columns, default_order);
+
+	g_strfreev (default_columns);
+	g_strfreev (default_order);
 }
 
 static GtkWidget *
@@ -2218,7 +2273,7 @@ create_column_editor (FMListView *view)
 	gtk_widget_show (alignment);
 	gtk_box_pack_start (GTK_BOX (box), alignment, TRUE, TRUE, 0);
 
-	column_chooser = nautilus_column_chooser_new ();
+	column_chooser = nautilus_column_chooser_new (file);
 	gtk_widget_show (column_chooser);
 	gtk_container_add (GTK_CONTAINER (alignment), column_chooser);
 
@@ -2706,6 +2761,12 @@ fm_list_view_dispose (GObject *object)
 		list_view->details->renaming_file_activate_timeout = 0;
 	}
 
+	if (list_view->details->clipboard_handler_id != 0) {
+		g_signal_handler_disconnect (nautilus_clipboard_monitor_get (),
+		                             list_view->details->clipboard_handler_id);
+		list_view->details->clipboard_handler_id = 0;
+	}
+
 	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
@@ -2828,6 +2889,31 @@ list_view_scroll_to_file (NautilusView *view,
 	}
 }
 
+static void
+list_view_notify_clipboard_info (NautilusClipboardMonitor *monitor,
+                                 NautilusClipboardInfo *info,
+                                 FMListView *view)
+{
+	if (info != NULL && info->cut) {
+		fm_list_model_set_highlight_for_files (view->details->model, info->files);
+	} else {
+		fm_list_model_set_highlight_for_files (view->details->model, NULL);
+	}
+}
+
+static void
+fm_list_view_end_loading (FMDirectoryView *view,
+                          gboolean all_files_seen)
+{
+	NautilusClipboardMonitor *monitor;
+	NautilusClipboardInfo *info;
+
+	monitor = nautilus_clipboard_monitor_get ();
+	info = nautilus_clipboard_monitor_get_clipboard_info (monitor);
+
+	list_view_notify_clipboard_info (monitor, info, FM_LIST_VIEW (view));
+}
+
 static void
 real_set_is_active (FMDirectoryView *view,
 		    gboolean is_active)
@@ -2862,6 +2948,7 @@ fm_list_view_class_init (FMListViewClass *class)
 
 	fm_directory_view_class->add_file = fm_list_view_add_file;
 	fm_directory_view_class->begin_loading = fm_list_view_begin_loading;
+	fm_directory_view_class->end_loading = fm_list_view_end_loading;
 	fm_directory_view_class->bump_zoom_level = fm_list_view_bump_zoom_level;
 	fm_directory_view_class->can_zoom_in = fm_list_view_can_zoom_in;
 	fm_directory_view_class->can_zoom_out = fm_list_view_can_zoom_out;
@@ -2957,6 +3044,10 @@ fm_list_view_init (FMListView *list_view)
 	list_view->details->zoom_level = NAUTILUS_ZOOM_LEVEL_SMALLEST - 1;
 
 	list_view->details->hover_path = NULL;
+	list_view->details->clipboard_handler_id = 
+		g_signal_connect (nautilus_clipboard_monitor_get (),
+		                  "clipboard_info",
+		                  G_CALLBACK (list_view_notify_clipboard_info), list_view);
 }
 
 static NautilusView *
