@@ -29,13 +29,17 @@
 
 #include <libnautilus-private/nautilus-entry.h>
 
-#define NAUTILUS_TYPE_LOCATION_ENTRY (nautilus_location_entry_get_type ())
+#define NAUTILUS_TYPE_LOCATION_ENTRY nautilus_location_entry_get_type()
 #define NAUTILUS_LOCATION_ENTRY(obj) \
-	GTK_CHECK_CAST (obj, NAUTILUS_TYPE_LOCATION_ENTRY, NautilusLocationEntry)
+  (G_TYPE_CHECK_INSTANCE_CAST ((obj), NAUTILUS_TYPE_LOCATION_ENTRY, NautilusLocationEntry))
 #define NAUTILUS_LOCATION_ENTRY_CLASS(klass) \
-	GTK_CHECK_CLASS_CAST (klass, NAUTILUS_TYPE_LOCATION_ENTRY, NautilusLocationEntryClass)
+  (G_TYPE_CHECK_CLASS_CAST ((klass), NAUTILUS_TYPE_LOCATION_ENTRY, NautilusLocationEntryClass))
 #define NAUTILUS_IS_LOCATION_ENTRY(obj) \
-	GTK_CHECK_TYPE (obj, NAUTILUS_TYPE_LOCATION_ENTRY)
+  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), NAUTILUS_TYPE_LOCATION_ENTRY))
+#define NAUTILUS_IS_LOCATION_ENTRY_CLASS(klass) \
+  (G_TYPE_CHECK_CLASS_TYPE ((klass), NAUTILUS_TYPE_LOCATION_ENTRY))
+#define NAUTILUS_LOCATION_ENTRY_GET_CLASS(obj) \
+  (G_TYPE_INSTANCE_GET_CLASS ((obj), NAUTILUS_TYPE_LOCATION_ENTRY, NautilusLocationEntryClass))
 
 typedef struct NautilusLocationEntryDetails NautilusLocationEntryDetails;
 
@@ -48,9 +52,16 @@ typedef struct {
 	NautilusEntryClass parent_class;
 } NautilusLocationEntryClass;
 
+typedef enum {
+	NAUTILUS_LOCATION_ENTRY_ACTION_GOTO,
+	NAUTILUS_LOCATION_ENTRY_ACTION_CLEAR
+} NautilusLocationEntryAction;
+
 GType      nautilus_location_entry_get_type     	(void);
 GtkWidget* nautilus_location_entry_new          	(void);
 void       nautilus_location_entry_set_special_text     (NautilusLocationEntry *entry,
 							 const char            *special_text);
+void       nautilus_location_entry_set_secondary_action (NautilusLocationEntry *entry,
+							 NautilusLocationEntryAction secondary_action);
 
 #endif /* NAUTILUS_LOCATION_ENTRY_H */
