@@ -700,7 +700,7 @@ clicked_eject_button (NautilusPlacesSidebar *sidebar,
 
 	*path = NULL;
 
-	if (event->type == GDK_BUTTON_PRESS &&
+	if ((event->type == GDK_BUTTON_PRESS || event->type == GDK_BUTTON_RELEASE) &&
 	    gtk_tree_view_get_path_at_pos (sidebar->tree_view,
 					   button_event->x, button_event->y,
 					   path, &column, NULL, NULL)) {
@@ -1043,18 +1043,16 @@ bookmarks_drop_uris (NautilusPlacesSidebar *sidebar,
 		nautilus_file_unref (file);
 
 		name = nautilus_compute_title_for_location (location);
-
 		icon = g_themed_icon_new (NAUTILUS_ICON_FOLDER);
-		bookmark = nautilus_bookmark_new_with_icon (location, name,
-							    FALSE, icon);
-		g_object_unref (icon);
-		
+		bookmark = nautilus_bookmark_new (location, name, TRUE, icon);
+
 		if (!nautilus_bookmark_list_contains (sidebar->bookmarks, bookmark)) {
 			nautilus_bookmark_list_insert_item (sidebar->bookmarks, bookmark, position++);
 		}
 
 		g_object_unref (location);
 		g_object_unref (bookmark);
+		g_object_unref (icon);
 		g_free (name);
 		g_free (uri);
 	}
@@ -2356,11 +2354,18 @@ bookmarks_button_release_event_cb (GtkWidget *widget,
 	GtkTreePath *path;
 	GtkTreeModel *model;
 	GtkTreeView *tree_view;
+	gboolean ret;
 
 	if (event->type != GDK_BUTTON_RELEASE) {
 		return TRUE;
 	}
 
+	if (clicked_eject_button (sidebar, &path)) {
+		ret = eject_or_unmount_bookmark (sidebar, path);
+		gtk_tree_path_free (path);
+		return ret;
+	}
+
 	tree_view = GTK_TREE_VIEW (widget);
 	model = gtk_tree_view_get_model (tree_view);
 
@@ -2390,20 +2395,11 @@ bookmarks_button_press_event_cb (GtkWidget             *widget,
 				 GdkEventButton        *event,
 				 NautilusPlacesSidebar *sidebar)
 {
-	GtkTreePath *path;
-	gboolean ret;
-
 	if (event->type != GDK_BUTTON_PRESS) {
 		/* ignore multiple clicks */
 		return TRUE;
 	}
 
-	if (clicked_eject_button (sidebar, &path)) {
-		ret = eject_or_unmount_bookmark (sidebar, path);
-		gtk_tree_path_free (path);
-		return ret;
-	}
-
 	if (event->button == 3) {
 		bookmarks_popup_menu (sidebar, event);
 	} else if (event->button == 2) {
@@ -2456,7 +2452,6 @@ bookmarks_edited (GtkCellRenderer       *cell,
 	bookmark = nautilus_bookmark_list_item_at (sidebar->bookmarks, index);
 
 	if (bookmark != NULL) {
-		nautilus_bookmark_set_has_custom_name (bookmark, TRUE);
 		nautilus_bookmark_set_name (bookmark, new_text);
 	}
 }
@@ -2668,6 +2663,11 @@ nautilus_places_sidebar_dispose (GObject *object)
 		sidebar->volume_monitor = NULL;
 	}
 
+	if (sidebar->bookmarks != NULL) {
+		g_object_unref (sidebar->bookmarks);
+		sidebar->bookmarks = NULL;
+	}
+
 	eel_remove_weak_pointer (&(sidebar->go_to_after_mount_slot));
 
 	G_OBJECT_CLASS (nautilus_places_sidebar_parent_class)->dispose (object);
@@ -2732,7 +2732,7 @@ nautilus_places_sidebar_set_parent_window (NautilusPlacesSidebar *sidebar,
 
 	slot = nautilus_window_info_get_active_slot (window);
 
-	sidebar->bookmarks = nautilus_window_info_get_bookmark_list (window);
+	sidebar->bookmarks = nautilus_bookmark_list_new ();
 	sidebar->uri = nautilus_window_slot_info_get_current_location (slot);
 
 	g_signal_connect_object (sidebar->bookmarks, "contents_changed",
