@@ -25,8 +25,9 @@
 #ifndef NAUTILUS_WINDOW_SLOT_H
 #define NAUTILUS_WINDOW_SLOT_H
 
-#include "nautilus-window.h"
+#include "nautilus-window-pane.h"
 #include "nautilus-query-editor.h"
+#include <glib/gi18n.h>
 
 #define NAUTILUS_TYPE_WINDOW_SLOT	 (nautilus_window_slot_get_type())
 #define NAUTILUS_WINDOW_SLOT_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST((k), NAUTILUS_TYPE_WINDOW_SLOT, NautilusWindowSlotClass))
@@ -64,7 +65,7 @@ struct NautilusWindowSlotClass {
 struct NautilusWindowSlot {
 	GObject parent;
 
-	NautilusWindow *window;
+	NautilusWindowPane *pane;
 
 	/* content_box contains
  	 *  1) an event box containing extra_location_widgets
@@ -107,6 +108,8 @@ struct NautilusWindowSlot {
 	gboolean tried_mount;
 
 	GCancellable *find_mount_cancellable;
+
+	gboolean visible;
 };
 
 GType   nautilus_window_slot_get_type (void);
@@ -142,6 +145,11 @@ const char	       *nautilus_window_slot_get_content_view_id      (NautilusWindow
 gboolean		nautilus_window_slot_content_view_matches_iid (NautilusWindowSlot	*slot,
 								       const char		*iid);
 
+void                    nautilus_window_slot_connect_content_view     (NautilusWindowSlot       *slot,
+								       NautilusView             *view);
+void                    nautilus_window_slot_disconnect_content_view  (NautilusWindowSlot       *slot,
+								       NautilusView             *view);
+
 #define nautilus_window_slot_go_to(slot,location, new_tab) \
 	nautilus_window_slot_open_location_full(slot, location, NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE, \
 						(new_tab ? NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB : 0), \
@@ -170,4 +178,6 @@ void    nautilus_window_slot_remove_extra_location_widgets (NautilusWindowSlot *
 
 void    nautilus_window_slot_add_current_location_to_history_list (NautilusWindowSlot *slot);
 
+void    nautilus_window_slot_is_in_active_pane (NautilusWindowSlot *slot, gboolean is_active);
+
 #endif /* NAUTILUS_WINDOW_SLOT_H */
